@@ -1326,11 +1326,40 @@ impl AsFormat for &ControlDirFormat {
 /// # Returns
 ///
 /// The newly created branch, or an error if the branch could not be created.
+#[deprecated(
+    since = "0.7.7",
+    note = "Use `create_branch_convenience_as_generic` instead to avoid unnecessary boxing"
+)]
 pub fn create_branch_convenience(
     base: &url::Url,
     force_new_tree: Option<bool>,
     format: impl AsFormat,
 ) -> Result<Box<dyn Branch>, Error> {
+    create_branch_convenience_as_generic(base, force_new_tree, format)
+        .map(|b| Box::new(b) as Box<dyn Branch>)
+}
+
+/// Create a branch conveniently, returning a GenericBranch.
+///
+/// This is similar to `create_branch_convenience`, but returns a `GenericBranch` directly
+/// instead of boxing it as a trait object. This is more efficient and allows the caller
+/// to use `GenericBranch`-specific methods and traits like `Clone`.
+///
+/// # Parameters
+///
+/// * `base` - The URL to create the branch at.
+/// * `force_new_tree` - Whether to force the creation of a new working tree if
+///   one already exists.
+/// * `format` - The format to use for the new branch.
+///
+/// # Returns
+///
+/// The newly created branch, or an error if the branch could not be created.
+pub fn create_branch_convenience_as_generic(
+    base: &url::Url,
+    force_new_tree: Option<bool>,
+    format: impl AsFormat,
+) -> Result<GenericBranch, Error> {
     Python::attach(|py| {
         let m = py.import("breezy.controldir")?;
         let cd = m.getattr("ControlDir")?;
@@ -1347,7 +1376,7 @@ pub fn create_branch_convenience(
             (base.to_string(),),
             Some(&kwargs),
         )?;
-        Ok(Box::new(GenericBranch::from(branch.unbind())) as Box<dyn Branch>)
+        Ok(GenericBranch::from(branch.unbind()))
     })
 }
 
@@ -1563,7 +1592,7 @@ mod tests {
     fn test_open_tree_or_branch() {
         crate::init();
         let tmp_dir = tempfile::tempdir().unwrap();
-        create_branch_convenience(
+        create_branch_convenience_as_generic(
             &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
             None,
             &ControlDirFormat::default(),
@@ -1700,7 +1729,7 @@ mod tests {
     fn test_create_branch_convenience() {
         crate::init();
         let tmp_dir = tempfile::tempdir().unwrap();
-        let branch = create_branch_convenience(
+        let branch = create_branch_convenience_as_generic(
             &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
             None,
             &ControlDirFormat::default(),
