@@ -76,8 +76,10 @@ impl std::str::FromStr for Vendor {
     }
 }
 
-impl FromPyObject<'_> for Vendor {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Vendor {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let vendor = ob.extract::<String>()?;
         match vendor.as_str() {
             "debian" => Ok(Vendor::Debian),
@@ -144,8 +146,10 @@ impl std::fmt::Display for VersionKind {
     }
 }
 
-impl FromPyObject<'_> for VersionKind {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for VersionKind {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let kind = ob.extract::<String>()?;
         match kind.as_str() {
             "auto" => Ok(VersionKind::Auto),
@@ -205,8 +209,10 @@ impl From<TarballKind> for Option<String> {
     }
 }
 
-impl FromPyObject<'_> for TarballKind {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for TarballKind {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let kind = ob.extract::<Option<String>>()?;
         Ok(kind.into())
     }
@@ -247,7 +253,7 @@ pub fn build_helper(
 ) -> Result<HashMap<String, PathBuf>, DebianError> {
     pyo3::prepare_freethreaded_python();
 
-    Python::with_gil(|py| -> PyResult<HashMap<String, PathBuf>> {
+    Python::attach(|py| -> PyResult<HashMap<String, PathBuf>> {
         let locals = PyDict::new(py);
         locals.set_item("local_tree", local_tree.to_object(py))?;
         locals.set_item("subpath", subpath.to_string_lossy().to_string())?;
@@ -283,7 +289,7 @@ pub fn tree_debian_tag_name(
     subpath: Option<&std::path::Path>,
     vendor: Option<Vendor>,
 ) -> Result<String, Error> {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let result = py.import("breezy.plugins.debian")?.call_method1(
             "tree_debian_tag_name",
             (
@@ -312,7 +318,7 @@ pub fn tree_debian_tag_name(
 /// # Returns
 /// Vendor or None if the distribution cannot be inferred.
 pub fn suite_to_distribution(suite: &str) -> Option<Vendor> {
-    Python::with_gil(|py| -> PyResult<Option<Vendor>> {
+    Python::attach(|py| -> PyResult<Option<Vendor>> {
         let result = py
             .import("breezy.plugins.debian.util")?
             .call_method1("suite_to_distribution", (suite,))?;
